using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Cursed.view;
using Cursed.gob;
using Cursed.gob.mob;
using Cursed.input;
using Cursed.gob.tile;
using Cursed.gob.mob.kob;
using Cursed.sound;
using Cursed.editor;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using Cursed.input.ai; 
using Cursed.view.renders;
using System.Windows;
using Cursed.col;


namespace Cursed
{

    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CursedGame : Microsoft.Xna.Framework.Game
    {
        public const float gameViewSize = 12;

        Profiler profiler;
        bool f1Down;

        GraphicsDeviceManager graphics;
        
        SpriteBatch spriteBatch;

        OrthoCamera camera;
        //CameraDetector cameraDetector;

        HUD Hud;

        Dictionary<Render.RenderType, Render> renders;
        ICollection<Gob> gobsToAdd;
        ICollection<Gob> gobsToRemove;
        ICollection<Gob> gobs;
        ICollection<Mob> mobs;
        ICollection<Tile> tiles;
        Dictionary<string, List<Gob>> labeledGobs;
        ICollection<Controller> controllers;
        ICollider collider;
        Cursed.sound.SoundBank soundBank;
        SoundManager soundManager;

        #region temporary
        public static Color FogColor = new Color(.2f, .22f, .2f);
        #endregion

        Effect effect;

        public static TextureLib textureLib;

        bool paused;
        Random random;

        public CursedGame()
        {
            profiler = new Profiler(true);

            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            Components.Add(new GamerServicesComponent(this));
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {

            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferredBackBufferHeight = 600;
            graphics.IsFullScreen = false;
            graphics.ApplyChanges();
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;
            Window.Title = "Cursed!";

            renders = new Dictionary<Render.RenderType, Render>();
            gobs = new List<Gob>();
            mobs = new List<Mob>();
            tiles = new List<Tile>();
            labeledGobs = new Dictionary<string, List<Gob>>();
            controllers = new List<Controller>();

            gobsToAdd = new List<Gob>();
            gobsToRemove = new List<Gob>();

            paused = false;
            random = new Random();

            base.Initialize();
        }

        public Render getRender(Render.RenderType type)
        {
            return renders[type];
        }

        void clearAllObjects()
        {
            gobs.Clear();
            mobs.Clear();
            tiles.Clear();
            labeledGobs.Clear();
        }

        public void LoadGameState(string path)
        {
            Stream stream = File.Open(path, FileMode.Open);
            BinaryFormatter bFormatter = new BinaryFormatter();
            List<GobInfo> gobInfos = (List<GobInfo>)bFormatter.Deserialize(stream);
            stream.Close();


            foreach (GobInfo gobInfo in gobInfos)
            {

            }
        }

        public void SaveGameState(string path)
        {
            List<GobInfo> gobInfos = new List<GobInfo>();
            foreach (Gob gob in gobs)
            {
                gobInfos.Add(gob.GobInfo);
            }

            Stream stream = File.Open(path, FileMode.Create);
            BinaryFormatter bFormatter = new BinaryFormatter();
            bFormatter.Serialize(stream, gobInfos);
            stream.Close();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            camera = new OrthoCamera(gameViewSize, Window);
            /*
            CameraDetectorInfo cdi = new CameraDetectorInfo();
            cdi.Width = camera.ViewWidth;
            cdi.Height = camera.ViewHeight;
            cameraDetector = new CameraDetector(this, cdi);
            AddGob(cameraDetector);
            */
            effect = Content.Load<Effect>("effects/BasicEffect");

            textureLib = new TextureLib(Content);
            
            SpriteRender sr = new SpriteRender(GraphicsDevice, effect);
            renders[Render.RenderType.SpriteRender] = sr;

            TrailRender tr = new TrailRender(effect, GraphicsDevice);
            renders[Render.RenderType.TrailRender] = tr;


            //collider = new TileCollider(this);
            collider = new QTCollider();


            Hud = new HUD();
            loadANewWorld();
            

            soundBank = new Cursed.sound.SoundBank(Content);
            soundManager = new SoundManager();
            new Editor(this, Vector2.Zero);
        }
        public Cursed.sound.SoundBank SoundBank
        {
            get { return soundBank; }
        }
        public SoundManager SoundManager
        {
            get { return soundManager; }
        }
        public ICamera Camera
        {
            get { return camera; }
        }

        public Random Random
        {
            get { return random; }
        }

        public ICollection<Gob> Gobs
        {
            get { return gobs; }
        }

        public ICollection<Mob> Mobs
        {
            get { return mobs; }
        }

        public ICollection<Tile> Tiles
        {
            get { return tiles; }
        }

        public void AddController(Controller controller)
        {
            controllers.Add(controller);
        }

        public void RemoveController(Controller controller)
        {
            controllers.Remove(controller);
            controller.cleanup();
        }

        public void AddGob(Gob gob)
        {
            gobsToAdd.Add(gob);
        }

        public void RemoveGob(Gob gob)
        {
            gobsToRemove.Add(gob);
            gob.cleanup();
        }

        void updateGobList()
        {
            if (gobsToAdd.Count == 0 && gobsToRemove.Count == 0)
                return;
            foreach (Gob gob in gobsToAdd)
            {
                if (gob is Tile)
                {
                    tiles.Add((Tile)gob);
                    collider.AddTile((Tile)gob);
                }

                if (gob is Mob)
                {
                    mobs.Add((Mob)gob);
                    collider.AddMob((Mob)gob);
                }
                if (gob.GobInfo.Label != null)
                {
                    if (!labeledGobs.ContainsKey(gob.GobInfo.Label))
                    {
                        labeledGobs[gob.GobInfo.Label] = new List<Gob>();
                    }
                    labeledGobs[gob.GobInfo.Label].Add(gob);
                }

                gobs.Add(gob);
            }
            gobsToAdd.Clear();

            foreach (Gob gob in gobsToRemove)
            {
                if (gob is Tile)
                {
                    tiles.Remove((Tile)gob);
                    collider.RemoveTile((Tile)gob);
                }

                if (gob is Mob)
                {
                    mobs.Remove((Mob)gob);
                    collider.RemoveMob((Mob)gob);
                }

                if (gob.GobInfo.Label != null)
                {
                    if (labeledGobs.ContainsKey(gob.GobInfo.Label))
                    {
                        labeledGobs[gob.GobInfo.Label].Remove(gob);
                    }
                }

                gobs.Remove(gob);
            }
            gobsToRemove.Clear();
        }

        public ICollection<Gob> getLabeledGobs(string label)
        {
            if (!labeledGobs.ContainsKey(label))
            {
                labeledGobs[label] = new List<Gob>();
            }
            return labeledGobs[label];
        }

        public ICollection<Mob> MobsInBounds(Rect bounds)
        {
            return collider.MobsInBounds(bounds);
        }

        void loadANewWorld()
        {
            DruidInfo di = new DruidInfo();
            Druid d = new Druid(this, di);

            Controller keyController = new KeyDirectionController();
            keyController.Push(d);
            controllers.Add(keyController);

            Controller trailController = new MouseTrailController(camera, Window);
            trailController.Push(d);
            AddController(trailController);

            Controller mouseController = new MouseController(camera, Window);
            mouseController.Push(d);
            AddController(mouseController);

            camera.PushFocus(d);
            Hud.AddComponenet(new HealthBar(new Rectangle(0, 0, 200, 20), d));
            Hud.AddComponenet(new ManaBar(new Rectangle(0, 20, 200, 20), d));
            AddGob(d);

            FamiliarInfo fmi = new FamiliarInfo();
            fmi.Loc = new Vector2(0, 1);
            Familiar fmr = new Familiar(this, fmi);
            AddGob(fmr);

            //first room
            for (int x = -3; x <= 3; x++)
            {
                for (int y = -3; y <= 3; y++)
                {
                    GroundInfo gi = new GroundInfo(x, y);
                    Ground g = new Ground(this, gi);
                    AddGob(g);
                }
            }
            for (int y = -3; y <= 3; y++)
            {
                WallInfo wi = new WallInfo(-4, y);
                Wall w = new Wall(this, wi);
                AddGob(w);
                wi = new WallInfo(4, y);
                w = new Wall(this, wi);
                AddGob(w);
            }
            for (int x = -4; x <= 4; x++)
            {
                WallInfo wi = new WallInfo(x, -4);
                Wall w = new Wall(this, wi);
                AddGob(w);
                if (x < -2 || x > 2)
                {
                    wi = new WallInfo(x, 4);
                    w = new Wall(this, wi);
                    AddGob(w);
                }
            }

            //second room
            for (int y = 4; y < 10; y++)
            {
                WallInfo wi = new WallInfo(-2, y);
                Wall w = new Wall(this, wi);
                AddGob(w);
                wi = new WallInfo(2, y);
                w = new Wall(this, wi);
                AddGob(w);
                for (int x = -1; x <= 1; x++)
                {
                    GroundInfo gi = new GroundInfo(x, y);
                    Ground g = new Ground(this, gi);
                    AddGob(g);
                }
            }
            for (int y = 10; y < 13; y++)
            {
                WallInfo wi = new WallInfo(-2, y);
                Wall w = new Wall(this, wi);
                AddGob(w);
                for (int x = -1; x <= 2; x++)
                {
                    GroundInfo gi = new GroundInfo(x, y);
                    Ground g = new Ground(this, gi);
                    AddGob(g);
                }
            }
            for (int x = -2; x <= 2; x++)
            {
                if (x == 0)
                {
                    GroundInfo gi = new GroundInfo(x, 13);
                    Ground g = new Ground(this, gi);
                    AddGob(g);
                }
                else
                {
                    WallInfo wi = new WallInfo(x, 13);
                    Wall w = new Wall(this, wi);
                    AddGob(w);
                }
            }
            for (int y = 9; y <= 13; y++)
            {
                for (int x = 2; x < 4; x++)
                {
                    if (y == 11 || x == 2)
                    {
                        GroundInfo gi = new GroundInfo(x, y);
                        Ground g = new Ground(this, gi);
                        AddGob(g);
                    }
                    else
                    {
                        WallInfo wi = new WallInfo(x, y);
                        Wall w = new Wall(this, wi);
                        AddGob(w);
                    }
                }
            }

            //room 3
            for (int y= 14; y <= 16; y++)
                for (int x = -1; x <= 1; x++)
                {
                    GroundInfo gi = new GroundInfo(x, y);
                    Ground g = new Ground(this, gi);
                    AddGob(g);
                }
            for (int y = 14; y <= 17; y++)
            {
                WallInfo wi = new WallInfo(-2, y);
                Wall w = new Wall(this, wi);
                AddGob(w);
                wi = new WallInfo(2, y);
                w = new Wall(this, wi);
                AddGob(w);
            }
            for (int x = -2; x <= 2; x++)
            {
                WallInfo wi = new WallInfo(x, 17);
                Wall w = new Wall(this, wi);
                AddGob(w);
            }

            //room 4
            for (int x = 3; x <= 12; x++)
                for (int y = 10; y <= 12; y++)
                {
                    GroundInfo gi = new GroundInfo(x, y);
                    Ground g = new Ground(this, gi);
                    AddGob(g);
                }
            for (int x = 6; x <= 12; x++)
                for (int y = 13; y <= 18; y++)
                {
                    GroundInfo gi = new GroundInfo(x, y);
                    Ground g = new Ground(this, gi);
                    AddGob(g);
                }
            for (int x = 3; x <= 13; x++)
            {
                WallInfo wi = new WallInfo(x, 9);
                Wall w = new Wall(this, wi);
                AddGob(w);
            }
            for (int y = 10; y <= 19; y++)
            {
                if (y == 13)
                    continue;
                WallInfo wi = new WallInfo(13, y);
                Wall w = new Wall(this, wi);
                AddGob(w);
            }
            for (int x = 6; x <= 12; x++)
            {
                WallInfo wi = new WallInfo(x, 19);
                Wall w = new Wall(this, wi);
                AddGob(w);
            }
            for (int y = 13; y <= 19; y++)
            {
                WallInfo wi = new WallInfo(5, y);
                Wall w = new Wall(this, wi);
                AddGob(w);
            }
            WallInfo wi2 = new WallInfo(4, 13);
            Wall w2 = new Wall(this, wi2);
            AddGob(w2);

            //room 5
            for (int x = 14; x <= 24; x++)
            {
                for (int y = 7; y < 22; y++)
                {
                    AddGob(new Ground(this, new GroundInfo(x, y)));
                }
                AddGob(new Wall(this, new WallInfo(x, 6)));
                AddGob(new Wall(this, new WallInfo(x, 22)));
            }
            for (int y = 6; y <= 22; y++)
            {
                if (y < 12 || y > 14)
                {
                    AddGob(new Wall(this, new WallInfo(25, y)));
                    AddGob(new Wall(this, new WallInfo(13, y)));
                }
            }
            for (int x = 25; x < 35; x++)
            {
                AddGob(new Wall(this, new WallInfo(x, 11)));
                AddGob(new Wall(this, new WallInfo(x, 15)));
                for (int y = 12; y <= 14; y++)
                {
                    AddGob(new Ground(this, new GroundInfo(x, y)));
                }
            }
            for (int y = 11; y <= 15; y++)
            {
                AddGob(new Wall(this, new WallInfo(35, y)));
            }
            for (int x = 18; x <= 20; x++)
            {
                for (int y = 15; y < 17; y++)
                {
                    AddGob(new Wall(this, new WallInfo(x, y)));
                }
                for (int y = 10; y < 12; y++)
                {
                    AddGob(new Wall(this, new WallInfo(x, y)));
                }
            }

            FootSwitchInfo switchI = new FootSwitchInfo(0, 14);
            switchI.Label = "door1";
            AddGob(new FootSwitch(this, switchI));

            DoorInfo doorI = new DoorInfo(0, 13);
            doorI.Label = "door1";
            AddGob(new Door(this, doorI));

            doorI = new DoorInfo(3, 11);
            doorI.Label = "door2";
            AddGob(new Door(this, doorI));

            switchI = new FootSwitchInfo(-1, 11);
            switchI.Label = "door2";
            AddGob(new FootSwitch(this, switchI));

            doorI = new DoorInfo(13, 13);
            doorI.Label = "door3";
            AddGob(new Door(this, doorI));

            switchI = new FootSwitchInfo(12, 13);
            switchI.Label = "door3";
            AddGob(new FootSwitch(this, switchI));

            
            DemonInfo dmi = new DemonInfo();
            dmi.Loc = new Vector2(0, 8);
            AddGob(new Demon(this, dmi));

            dmi = new DemonInfo();
            dmi.Transparency = .1f;
            dmi.Scale = .5f;
            dmi.Loc = new Vector2(-1, 16);
            AddGob(new Demon(this, dmi));

            dmi = new DemonInfo();
            dmi.Transparency = .1f;
            dmi.Scale = .5f;
            dmi.Loc = new Vector2(1, 16);
            AddGob(new Demon(this, dmi));

            dmi = new DemonInfo();
            dmi.Transparency = .1f;
            dmi.Scale = .5f;
            dmi.Loc = new Vector2(0, 16);
            AddGob(new Demon(this, dmi));

            for (int x = 6; x <= 12; x++)
                for (int y = 15; y <= 18; y++)
                {
                    dmi = new DemonInfo();
                    dmi.Loc = new Vector2(x, y);
                    dmi.Scale = .5f;
                    AddGob(new Demon(this, dmi));
                }

            dmi = new DemonInfo();
            dmi.MaxHealth = 5;
            dmi.CurrentHealth = 5;
            dmi.Lunge = true;
            dmi.SpinAttack = true;
            dmi.Scale = 2;
            dmi.AccMag = 8;
            dmi.StrengthFactor = 2;
            dmi.ViewRange = 12;
            dmi.Loc = new Vector2(33, 13);
            AddGob(new Demon(this, dmi));
        }

        void loadAWorld()
        {

            DruidInfo di = new DruidInfo();
            Druid d = new Druid(this, di);

            Controller keyController = new KeyDirectionController();
            keyController.Push(d);
            controllers.Add(keyController);

            Controller trailController = new MouseTrailController(camera, Window);
            trailController.Push(d);
            AddController(trailController);

            Controller mouseController = new MouseController(camera, Window);
            mouseController.Push(d);
            AddController(mouseController);

            camera.PushFocus(d);
            Hud.AddComponenet(new HealthBar(new Rectangle(0, 0, 200, 20), d));
            Hud.AddComponenet(new ManaBar(new Rectangle(0, 20, 200, 20), d));
            AddGob(d);

            FamiliarInfo fmi = new FamiliarInfo();
            fmi.Loc = new Vector2(0, 1);
            Familiar fmr = new Familiar(this, fmi);
            AddGob(fmr);
            
            for (int x = 0; x < 9; x++)
            {
                for (int y = 0; y < 9; y++)
                {
                    GroundInfo gi = new GroundInfo(x-4, y-4);
                    Ground g = new Ground(this, gi);
                    AddGob(g);
                }
                WallInfo wi = new WallInfo(x - 4, -5);
                Wall w = new Wall(this, wi);
                AddGob(w); 
                wi = new WallInfo(x - 4, 5);
                w = new Wall(this, wi);
                AddGob(w);
            }
            for (int y = -1; y < 10; y++)
            {
                WallInfo wi = new WallInfo(-5, y - 4);
                Wall w = new Wall(this, wi);
                AddGob(w); wi = new WallInfo(5, y - 4);
                w = new Wall(this, wi);
                AddGob(w);
            }

            Door door = new Door(this, new DoorInfo(-3, -3));
            AddGob(door);
            FootSwitch sw = new FootSwitch(this, new FootSwitchInfo(-3, -2));
            AddGob(sw);

            DemonSpawnInfo dsi = new DemonSpawnInfo(3, -3);
            dsi.SpawnTime = 10;
            DemonSpawn ds = new DemonSpawn(this, dsi);
            AddGob(ds);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            if (!f1Down && Keyboard.GetState()[Keys.F1] == KeyState.Down)
            {
                profiler.Print();
            }
            f1Down = Keyboard.GetState()[Keys.F1] == KeyState.Down;
            if (Keyboard.GetState()[Keys.F2] == KeyState.Down)
            {
                profiler.Reset();
            }
            profiler.Tick();
            profiler.Start("Update");

            updateGobList();

            if (Paused)
                gameTime = new GameTime(TimeSpan.Zero, TimeSpan.Zero, TimeSpan.Zero, TimeSpan.Zero);

            foreach (Render render in renders.Values)
            {
                render.Reposition(camera);
            }

            foreach (Controller controller in controllers)
            {
                controller.act(gameTime);
            }

            profiler.Start("collider.handleCollisions");
            collider.handleCollisions();
            profiler.Stop("collider.handleCollisions");

            foreach (Gob gob in gobs)
            {
                gob.Update(gameTime);
            }

            camera.trackFocus();

            profiler.Stop("Update");

            base.Update(gameTime);
        }

        int compareGobDepth(Gob g1, Gob g2)
        {
            if (g1.GobInfo.Depth < g2.GobInfo.Depth)
                return 1;
            if (g1.GobInfo.Depth > g2.GobInfo.Depth)
                return -1;
            return 0;
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            profiler.Start("Draw");
            GraphicsDevice.Clear(Color.Black);
            GraphicsDevice.RenderState.AlphaBlendEnable = true;
            GraphicsDevice.RenderState.SourceBlend = Blend.SourceAlpha;
            GraphicsDevice.RenderState.DestinationBlend = Blend.InverseSourceAlpha;
            GraphicsDevice.RenderState.DepthBufferEnable = false;
            //GraphicsDevice.Clear(ClearOptions.Target | ClearOptions.DepthBuffer, Color.Black, 100.0f, 0);

            List<Gob> glist = (List<Gob>)gobs;
            Profiler.Start("glist.Sort");
            glist.Sort(compareGobDepth);
            Profiler.Stop("glist.Sort");

            Rect cameraBounds = camera.Bounds;

            foreach (Gob gob in gobs)//cameraDetector.TouchedGobs)
            {
                if (cameraBounds.IntersectsWith(gob.Bounds))
                    gob.Draw(gameTime);
            }
            
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Texture, SaveStateMode.SaveState);
            spriteBatch.Draw(textureLib.GetTexture("fog"),
                new Rectangle(
                    0,
                    0,
                    GraphicsDevice.Viewport.Width,
                    GraphicsDevice.Viewport.Height),
                    FogColor);
            spriteBatch.End();
            if (FogColor.R > 51.2)
            {
                FogColor = new Color((byte)(FogColor.R - 5), FogColor.G, FogColor.B);
            }
            Hud.Draw(spriteBatch, gameTime);
            
            profiler.Stop("Draw");

            base.Draw(gameTime);
        }

        public bool Paused
        {
            get { return paused; }
            set { paused = value; }
        }

        public Profiler Profiler
        {
            get { return profiler; }
        }
    }
}
