﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.view;
using Cursed.input;
using Microsoft.Xna.Framework;
using Cursed.gob;
using Cursed.geom;
using Cursed.view.renders;

namespace Cursed.editor
{
    class CursorInfo : GobInfo
    {
        public CursorInfo()
            : base()
        {
            Depth = -1; //on top
        }
    }
    class Cursor : Gob
    {
        Vector2 loc;
        RenderInfo renderInfo;

        public Cursor(CursedGame game)
            : base(game, new CursorInfo())
        {

            shape = new Rect(Vector2.Zero, .1f, .1f);
            renderInfo = new SpriteRenderInfo(Bounds);
            renderInfo.Mask = Microsoft.Xna.Framework.Graphics.Color.DarkOliveGreen;
            addRender(game.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);
        }

        public Vector2 Loc
        {
            get { return loc; }
            set
            {
                loc = value;
                Vector3 loc3 = new Vector3(loc.X, loc.Y, GobInfo.Depth);
                renderInfo.Transform = Microsoft.Xna.Framework.Matrix.CreateTranslation(loc3);
            }
        }
    }

    class Editor : IFocus, IControllable
    {
        CursedGame game;
        ICamera camera;
        Cursor cursor;
        Vector2 loc;
        MouseController mouseController;
        KeyDirectionController directionController;
        EscControl escapeControl;

        bool active;

        public Editor(CursedGame game, Vector2 loc)
        {
            this.game = game;
            camera = game.Camera;

            cursor = new Cursor(game);
            this.loc = loc;

            mouseController = new MouseController(camera, game.Window);
            game.AddController(mouseController);
            mouseController.Push(this);

            directionController = new KeyDirectionController();
            game.AddController(directionController);
            directionController.Push(this);

            escapeControl = new EscControl();
            game.AddController(escapeControl);
            escapeControl.Push(this);

            active = false;
        }

        public void activate()
        {
            game.AddGob(cursor);
            camera.PushFocus(this);
            game.Paused = true;
            active = true;
        }

        public void deactivate()
        {
            game.RemoveGob(cursor);
            camera.PopFocus(this);
            game.Paused = false;
            active = false;
        }

        #region IControllable Members

        public void control(ControlInfo info)
        {
            if (info is MouseCursorInfo)
            {
                MouseCursorInfo mouseCursorInfo = (MouseCursorInfo)info;
                cursor.Loc = mouseCursorInfo.Mouse;
                if (mouseCursorInfo.Clicked)
                {
                    //game.SaveGameState("test.cursed");
                }
            }
            if (info is DirectionControlInfo)
            {
                DirectionControlInfo directionControlInfo = (DirectionControlInfo)info;
                loc += directionControlInfo.Direction * .05f;
            }
            if (info is EscapeControlInfo)
            {
                if (active)
                    deactivate();
                else
                    activate();
            }
        }

        #endregion

        #region IFocus Members

        public Vector2 Loc
        {
            get { return loc; }
        }

        #endregion
    }
}
