﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Cursed.view;
using System.Windows.Media;
using System.Windows;
using Cursed.geom;
using System.Reflection;
using System.Runtime.Serialization;
using Cursed.view.renders;
using Cursed.gob.mob;

namespace Cursed.gob
{
    public class GobInfo
    {
        string label;
        Vector2 loc;
        // the lower the depth, the closer it is to the camera
        float viewRange;
        float depth;

        public GobInfo()
        {
            label = null;
            loc = Vector2.Zero;
            this.depth = 0;
            viewRange = 0;
        }

        public virtual Gob create(CursedGame game)
        {
            throw new NotImplementedException();
        }

        public float ViewRange
        {
            get { return viewRange; }
            set { viewRange = value; }
        }
        public string Label
        {
            get { return label; }
            set { label = value; }
        }

        public Vector2 Loc
        {
            get { return loc; }
            set { loc = value; }
        }

        public Vector3 Loc3
        {
            get { return new Vector3(loc.X, loc.Y, Depth); }
            set
            {
                loc.X = value.X;
                loc.Y = value.Y;
                Depth = value.Z;
            }
        }

        public float Depth
        {
            get { return depth; }
            set { depth = value; }
        }

    }

    public class RenderPair
    {
        public Render render;
        public RenderInfo info;
        public RenderPair(Render render, RenderInfo info)
        {
            this.render = render;
            this.info = info;
        }
    }

    public class Gob : DrawableGameComponent
    {
        
        protected GobInfo info;
        protected List<RenderPair> renders;
        
        // if something is transcendant, then it does not participate in physics collisions
        bool transcendant;
        //if transient, don't save/send 
        bool transient;

        /*
         * the zero-offset, zero-rotation shape used for collision detection.
         */
        protected IShape shape;
        protected ICollection<Gob> touchedGobs;
        ICollection<Mob> seenMobs;

        protected float coolDown;

        public Gob(Game game, GobInfo info)
            : base(game)
        {
            this.info = info;
            renders = new List<RenderPair>();
            touchedGobs = new List<Gob>();
            seenMobs = new List<Mob>();
            transcendant = true;
            transient = false;

            coolDown = 0;
        }

        public virtual bool touches(Gob other)
        {
            return Shape.intersects(other.Shape);
        }

        public void touch(Gob gob)
        {
            touchedGobs.Add(gob);
        }

        public void clearTouch()
        {
            touchedGobs.Clear();
        }
        protected virtual void updateSeenMobs()
        {
            CursedGame.Profiler.Start("updateSeenMobs");
            seenMobs.Clear();

            Circle c = new Cursed.geom.Circle(GobInfo.Loc, GobInfo.ViewRange);
            

            float vr2 = GobInfo.ViewRange * GobInfo.ViewRange;
            foreach (Mob mob in CursedGame.MobsInBounds(c.Bounds))
            {
                float distance2 = (mob.GobInfo.Loc - GobInfo.Loc).LengthSquared();
                if (distance2 < vr2)
                    seenMobs.Add(mob);
            }
            CursedGame.Profiler.Stop("updateSeenMobs");
        }
        public ICollection<Mob> SeenMobs
        {
            get { return seenMobs; }
        }

        public virtual void cleanup()
        {
            
        }

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);
            foreach (RenderPair pair in renders)
            {
                pair.render.Begin(pair.info);
                pair.render.Draw(gameTime, pair.info);
                pair.render.End();
            }
        }

        public override void Update(GameTime gameTime)
        {
            coolDown -= gameTime.ElapsedGameTime.Milliseconds * .001f;
            if (coolDown < 0)
                coolDown = 0;
            if (GobInfo.ViewRange > 0 && CursedGame.Random.NextDouble() < .1)
                updateSeenMobs();
        }

        public float CoolDown
        {
            get { return coolDown; }
            set { coolDown = value; }
        }

        public bool Ready
        {
            get { return coolDown <= 0; }
        }

        protected void addRender(Render render, RenderInfo info)
        {
            renders.Add(new RenderPair(render, info));
        }

        protected void clearRenders()
        {
            renders.Clear();
        }

        public ICollection<Gob> TouchedGobs
        {
            get { return touchedGobs; }
        }

        public CursedGame CursedGame
        {
            get { return (CursedGame)Game; }
        }

        public IShape Shape
        {
            get { return shape; }
        }

        public virtual bool Transcendent
        {
            get { return transcendant; }
            set { transcendant = value; }
        }

        public bool Transient
        {
            get { return transient; }
            set { transient = value; }
        }

        public System.Windows.Rect Bounds
        {
            get { return Shape.Bounds; }
        }

        public Vector2 BoundsCenter
        {
            get
            {
                Vector2 c = new Vector2((float)Bounds.X, (float)Bounds.Y);
                c.X += (float)Bounds.Width * .5f;
                c.Y += (float)Bounds.Height * .5f;
                return c;
            }
        }

        public GobInfo GobInfo
        {
            get { return info; }
        }
        
    }
}
