﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.gob.mob.kob;
using Microsoft.Xna.Framework;
using Cursed.geom;
using Cursed.view.renders;
using Microsoft.Xna.Framework.Graphics;
using Cursed.sound;

namespace Cursed.gob.mob
{
    public class AttackInfo : MobInfo
    {
        float strength;
        float energy;
        float duration;

        public AttackInfo()
            : base()
        {
            strength = 0;
            energy = 0;
            duration = 0;
            Depth = Const.AttackDepth;
        }

        public float Strength
        {
            get { return strength; }
            set { strength = value; }
        }

        public float Energy
        {
            get { return energy; }
            set { energy = value; }
        }

        public float Duration
        {
            get { return duration; }
            set { duration = value; }
        }
    }

    public class ClawsAttack : Attack
    {
        static String[] sounds = { "Spirit\\SpiritAttack1", "Spirit\\SpiritAttack1", "Spirit\\SpiritAttack1" };
        public ClawsAttack(CursedGame game, AttackInfo info, Gob source, IShape shape)
            : base(game, info, source, shape)
        {
            renderInfo = new SpriteRenderInfo(shape.Bounds);
            ((SpriteRenderInfo)renderInfo).Texture = CursedGame.textureLib.GetTexture("hit");
            ((SpriteRenderInfo)renderInfo).Mask = Color.White;
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);
            VariableSound snd = new VariableSound(game.SoundBank, game.SoundManager, sounds, game.Random);
            snd.play();
        }
    }
    public class SpinAttack : Attack
    {
        static string sound = "Friendly Spirit\\spinSound" ;
        public SpinAttack(CursedGame game, AttackInfo info, Gob source, IShape shape)
            : base(game, info, source, shape)
        {
            renderInfo = new SpriteRenderInfo(shape.Bounds);
            ((SpriteRenderInfo)renderInfo).Texture = CursedGame.textureLib.GetTexture("spin");
            ((SpriteRenderInfo)renderInfo).Mask = Color.White;
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);
            SimpleSound snd = new SimpleSound(game.SoundBank, game.SoundManager, sound);
            snd.play();
        }
        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);
            this.AttackInfo.Theta +=(float)Math.PI * 2 * (1 - gameTime.ElapsedGameTime.Milliseconds*.001f / AttackInfo.Duration);
        }
    }
    public class Attack : Mob
    {
        protected float timeLeft;
        ICollection<Kob> targets;
        Gob source;

        public Attack(CursedGame game, AttackInfo info, Gob source, IShape shape)
            : base(game, info)
        {
            this.source = source;
            this.shape = shape;
            renderInfo = null;
            targets = new HashSet<Kob>();
            timeLeft = AttackInfo.Duration;
            Transcendent = true;
            Transient = true;
        }

        public AttackInfo AttackInfo
        {
            get { return (AttackInfo)info; }
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            timeLeft -= gameTime.ElapsedGameTime.Milliseconds * .001f;

            float alpha = timeLeft / AttackInfo.Duration;
            if (alpha < 0) alpha = 0;
            Color mask = ((SpriteRenderInfo)renderInfo).Mask;
            mask.A = (byte)(alpha * 255);
            ((SpriteRenderInfo)renderInfo).Mask = mask;

            foreach (Gob gob in TouchedGobs)
            {
                if (gob is Kob && gob != source)
                {
                    targets.Add((Kob)gob);
                }
            }

            if (timeLeft <= 0)
            {
                foreach (Kob target in targets)
                {
                    float attackFactor = 1;
                    if (target.KobInfo.Energy > 0 != AttackInfo.Energy > 0)
                        attackFactor = -1 * AttackInfo.Energy / target.KobInfo.Energy;
                    if (attackFactor > 1)
                        attackFactor = 1;
                    float damage = AttackInfo.Strength * attackFactor;
                    target.takeDamage(damage);
                }
                CursedGame.RemoveGob(this);
            }

            base.Update(gameTime);
        }
    }
}
