﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Cursed.geom;
using Cursed.view.renders;
using Cursed.gob.mob.kob;

namespace Cursed.gob.mob
{
     public class LifeBlobInfo : MobInfo
    {
        float amount;
        float sizeRatio;
        float decay;
        public LifeBlobInfo()
        {
            amount = .5f;
            decay = .2f;
            sizeRatio = 1f;
            Depth = Const.LifeBlobDepth;
        }

        public override Gob create(CursedGame game)
        {
            return new LifeBlob(game, this);
        }
        public float Amount
        {
            get { return amount; }
            set { amount = value; }
        }
        public float SizeRatio
        {
            get{return sizeRatio;}
            set{ sizeRatio = value;}
        }
        public float Decay
        {
            get { return decay; }
            set { decay = value; }
        }
    }

    class LifeBlob : Mob
    {
       
        public LifeBlob(CursedGame game, LifeBlobInfo info)
            : base(game, info)
        {
            shape = new Circle(Vector2.Zero, .5f);
            renderInfo = new SpriteRenderInfo(shape.Bounds);
            renderInfo.Mask = Microsoft.Xna.Framework.Graphics.Color.Green;
            renderInfo.Texture = CursedGame.textureLib.GetTexture("circle");
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);

            updateTransform();

            Transient = false;
            Transcendent = true;
        }
        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            
            LifeInfo.Amount -= LifeInfo.Decay * gameTime.ElapsedGameTime.Milliseconds / 1000f;


            int demonCount = 0;
            foreach (Gob gob in TouchedGobs)
            {
                if (gob is Demon)
                {
                    demonCount++;
                }
            }
            foreach (Gob gob in TouchedGobs)
            {
                if (gob is Demon)
                {
                    Demon demon = (Demon)gob;
                    float transfer = LifeInfo.Amount / demonCount;
                    transfer *= gameTime.ElapsedGameTime.Milliseconds * .001f;
                    if (!(gob is Familiar))
                        demon.DemonInfo.Energy += transfer;
                    LifeInfo.Amount -= transfer;
                }
            }

            if (LifeInfo.Amount <= 0)
            {
                CursedGame.RemoveGob(this);
            }
            else
                LifeInfo.Scale = LifeInfo.Amount * LifeInfo.SizeRatio;


            updateTransform();

            base.Update(gameTime);
        }
        public LifeBlobInfo LifeInfo
        {
            get { return (LifeBlobInfo)info; }
        }

    }
}
