﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.input.ai;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Cursed.geom;
using Cursed.input;
using Cursed.view.renders;
using Cursed.sound;

namespace Cursed.gob.mob.kob
{
    public class DemonInfo : KobInfo
    {
        int tailLength;
        float transparency;
        float decay;
        bool lunge;
        float lungeRange;
        bool spinAttack;
        float strength;

        public DemonInfo()
            : base()
        {
            ViewRange = 5;
            AccMag = 4;
            tailLength = 10;
            this.transparency = 1f;
            decay = .1f;
            Energy = -1f;
            Mass = Scale;
            Depth = Const.DemonDepth;
            lunge = false;
            spinAttack = false;
            lungeRange = 2f;
            strength = 1;
        }

        public override Gob create(CursedGame game)
        {
            return new Demon(game, this);
        }

        public float StrengthFactor
        {
            get { return strength; }
            set { strength = value; }
        }
        public float Transparency
        {
            get { return transparency; }
            set { transparency = value; }
        }
        public int TailLength
        {
            get { return tailLength; }
            set { tailLength = value; }

        }
        public float LungeRange
        {
            get { return lungeRange; }
            set { lungeRange = value; }
        }
        public float Decay
        {
            get { return decay;}
            set { decay = value; }
        }
        public bool SpinAttack
        {
            get { return spinAttack; }
            set { spinAttack = value; }
        }
        public bool Lunge
        {
            get { return lunge; }
            set { lunge = value; }
        }
    }

    public class Demon : Kob
    {
        public const float demonRadius = .5f;
        TrailRenderInfo trailInfo;    
        public static string[] dmgSounds = { "Spirit\\hitSpirit1", "Spirit\\hitSpirit2" };        
        public static string lungeSound = "Friendly Spirit\\leap";        
        protected string sightSound;        
        Mob goal;            
        bool lunge;
        public static string deathSound = "Spirit\\spiritDeath";
        public TrailRenderInfo TrailInfo
        {
            get { return trailInfo; }
            set { trailInfo = value; }
        }
        public Demon(CursedGame game, DemonInfo info)
            : base(game, info)
        {
            initialize();
            addControllers();
            sightSound = "Spirit//goalSighted";
            goal = null;
            lunge = false;
        }
        protected virtual void initialize()
        {
            shape = new Circle(Vector2.Zero, demonRadius);
            renderInfo = new SpriteRenderInfo(shape.Bounds);
            renderInfo.Mask = Microsoft.Xna.Framework.Graphics.Color.Gray;
            renderInfo.Texture = CursedGame.textureLib.GetTexture("circle");
            updateTransform();


            trailInfo = new TrailRenderInfo();

            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.TrailRender), trailInfo);
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);

            Transient = false;
            Transcendent = false;
        }
        protected virtual void addControllers()
        {
            DruidSeeker seekerControl = new DruidSeeker(this);
            CursedGame.AddController(seekerControl);
            seekerControl.Push(this);

            DruidAttacker attackerControl = new DruidAttacker(this);
            CursedGame.AddController(attackerControl);
            attackerControl.Push(this);
        }
        public override bool touches(Gob other)
        {
            //if(!lunge || other.Equals(goal))
                return base.touches(other);
            //return false;
        }
        protected void performBasicAttack(Vector2 direction)
        {
            if (Ready)
            {
                Vector2 loc = GobInfo.Loc + direction * (demonRadius+.25f);

                float theta = (float)Math.Acos(direction.X);
                if (direction.Y < 0) theta *= -1;
                theta -= MathHelper.PiOver2;

                AttackInfo ati = new AttackInfo();
                ati.Loc = loc;
                ati.Vel = MobInfo.Vel;
                ati.Theta = theta;
                ati.Energy = KobInfo.Energy;
                ati.Strength = Const.DemonClawStrength*DemonInfo.StrengthFactor;
                ati.Duration = Const.DemonClawAttackDuration;
                Circle c = new Circle(Vector2.Zero, .5f);
                Attack at = new ClawsAttack(CursedGame, ati, this, c);
                CursedGame.AddGob(at);
                CoolDown += Const.DemonClawAttackCoolDown;
            }
        }
        protected void performSpinAttack()
        {
            float theta = (float)Math.Acos(direction.X);
                if (direction.Y < 0) theta *= -1;
                theta -= MathHelper.PiOver2;
            if (Ready)
            {
                AttackInfo ati = new AttackInfo();
                ati.Loc = GobInfo.Loc;
                ati.Vel = MobInfo.Vel;
                ati.Theta = theta;
                ati.Energy = KobInfo.Energy;
                ati.Strength = Const.DemonSpinStrength * DemonInfo.StrengthFactor;
                ati.Duration = Const.DemonSpinAttackDuration;
                Circle c = new Circle(Vector2.Zero, this.KobInfo.Scale * .75f);
                Attack at = new SpinAttack(CursedGame, ati, this, c);
                CursedGame.AddGob(at);
                CoolDown += Const.DemonSpinAttackCoolDown;

            }
        }
        public override void  control(Cursed.input.ControlInfo cInfo)
        {
 	        base.control(cInfo);
            if (cInfo is BasicAttackInfo)
            {
                lungeOff();
                if(((BasicAttackInfo)cInfo).Type == BasicAttackInfo.ATTACKTYPE.NORMAL)
                    performBasicAttack(((BasicAttackInfo)cInfo).Direction);
                else if (((BasicAttackInfo)cInfo).Type == BasicAttackInfo.ATTACKTYPE.SPIN)
                {
                    performSpinAttack();
                }
            }
        }
        public override void Update(GameTime gameTime)
        {
            if (DemonInfo.Energy > -1f)
            {
                DemonInfo.Energy -= gameTime.ElapsedGameTime.Milliseconds/1000f * DemonInfo.Decay;
                if (DemonInfo.Energy > 0)
                {
                    DemonInfo.Energy = 0f;
                }
            }
            if (DemonInfo.Lunge &&goal != null && !lunge 
                && (goal.Loc - this.Loc).LengthSquared() > Math.Pow(DemonInfo.LungeRange, 2))
                lungeOn();
            updateColor();
            trailInfo.Update((SpriteRenderInfo)renderInfo);
            base.Update(gameTime);
        }
        public virtual void lungeOn()
        {
            if (lunge) return;
            lunge = true;
            DemonInfo.AccMag *= 2f;
            SimpleSound snd = new SimpleSound(CursedGame.SoundBank, CursedGame.SoundManager, lungeSound);
            snd.play();
        }
        public virtual void lungeOff()
        {
            if (!lunge) return;
            lunge = false;
            DemonInfo.AccMag /= 2;
        }
        public virtual void updateColor()
        {
            float val = (1 + DemonInfo.Energy);
            renderInfo.Mask = new Color((1 - val) * .3f, (1 - val) * .3f + val*.6f, (1 - val) * .3f, DemonInfo.Transparency); 
        }
        public DemonInfo DemonInfo
        {
            get { return (DemonInfo)info; }
        }

        public override void die()
        {
            SimpleSound snd = new SimpleSound(CursedGame.SoundBank, CursedGame.SoundManager, deathSound);
            snd.play();
            base.die();
        }
        public Mob Goal
        {
            get{return goal;}
            set
            {
                if (value != null)
                {
                    SimpleSound snd = new SimpleSound(CursedGame.SoundBank, CursedGame.SoundManager, sightSound);
                    snd.play();
                }
                goal = value;
            }
        }
        public override void takeDamage(float damage)
        {
            base.takeDamage(damage);
            VariableSound snd = new VariableSound(CursedGame.SoundBank, 
                CursedGame.SoundManager, dmgSounds, CursedGame.Random);
            snd.play();
        }
        public override void cleanup()
        {
           
            base.cleanup();
        }
    }
}
