﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Cursed.view;
using System.Windows.Media;
using System.Windows;
using Cursed.input;
using System.Runtime.Serialization;
using Cursed.geom;
using Cursed.view.renders;
using Cursed.gob.tile;
using Cursed.sound;

namespace Cursed.gob.mob.kob
{
    public class DruidWandInfo : MobInfo
    {
        public DruidWandInfo()
            : base()
        {
            Depth = Const.DruidWandDepth;
            Mass = 1;
        }
    }

    public class DruidWand : Mob
    {
        public DruidWand(CursedGame game, DruidWandInfo info)
            : base(game, info)
        {

            shape = new Circle(Vector2.Zero, .05f);
            renderInfo = new SpriteRenderInfo(shape.Bounds);
            renderInfo.Mask = Microsoft.Xna.Framework.Graphics.Color.Blue;
            renderInfo.Texture = CursedGame.textureLib.GetTexture("circle");
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);

            Transient = true;
            Transcendent = true;
        }
    }

    public class DruidInfo : KobInfo
    {
        float healthRegen;
        float currentMana;
        float maxMana;
        float manaRegen;

        public DruidInfo()
            : base()
        {
            AccMag = 20;
            Damping = 3;
            healthRegen = 0.1f;
            currentMana = 1;
            maxMana = 1;
            manaRegen = .1f;
            Depth = Const.DruidDepth;
        }

        public override Gob create(CursedGame game)
        {
            return new Druid(game, this);
        }

        public float HealthRegen
        {
            get { return healthRegen; }
            set { healthRegen = value; }
        }
        public float CurrentMana
        {
            get { return currentMana; }
            set { currentMana = value; }
        }
        public float MaxMana
        {
            get { return maxMana; }
            set { maxMana = value; }
        }
        public float ManaRegen
        {
            get { return manaRegen; }
            set { manaRegen = value; }

        }
    }
    
    public class Druid : Kob
    {
        DruidWand wand;
        float timeToNextPoop;
        static string[] dmgSounds = { "Druid\\hitDruid1", "Druid\\hitDruid2", "Druid\\hitDruid3" };
        static string woodSound = "Spells\\treeGrow";
        static string lifeSound = "Spells\\spell1";
        public Druid(CursedGame game, DruidInfo info)
            : base(game, info)
        {
            shape = new Circle(Vector2.Zero, .5f);
            renderInfo = new SpriteRenderInfo(shape.Bounds);
            renderInfo.Mask = Microsoft.Xna.Framework.Graphics.Color.Green;
            renderInfo.Texture = CursedGame.textureLib.GetTexture("circle");
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);

            updateTransform();
            Transcendent = false;
            
            DruidWandInfo wandInfo = new DruidWandInfo();
            wand = new DruidWand(CursedGame, wandInfo);
            CursedGame.AddGob(wand);

            timeToNextPoop = 0;
        }
        public DruidInfo DruidInfo
        {
            get { return (DruidInfo)info; }
        }
        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            if (DruidInfo.CurrentHealth < DruidInfo.MaxHealth)
            {
                DruidInfo.CurrentHealth = Math.Min(DruidInfo.MaxHealth,
                    DruidInfo.CurrentHealth + DruidInfo.HealthRegen * gameTime.ElapsedGameTime.Milliseconds/1000f);
            }
            if (DruidInfo.CurrentMana < DruidInfo.MaxMana)
            {
                DruidInfo.CurrentMana = Math.Min(DruidInfo.MaxMana,
                    DruidInfo.CurrentMana + DruidInfo.ManaRegen * gameTime.ElapsedGameTime.Milliseconds / 1000f);
            }

            timeToNextPoop -= gameTime.ElapsedGameTime.Milliseconds * .001f;
            if (timeToNextPoop < 0)
            {
                LifeBlobInfo info = new LifeBlobInfo();
                Vector2 loc = GobInfo.Loc;
                loc.X += ((float)CursedGame.Random.NextDouble() - .5f) * .5f;
                loc.Y += ((float)CursedGame.Random.NextDouble() - .5f) * .5f;
                info.Loc = loc;
                info.Decay = Const.DruidPoopDecay;
                info.Amount = Const.DruidPoopAmount;
                LifeBlob blob = new LifeBlob(CursedGame, info);
                CursedGame.AddGob(blob);
                timeToNextPoop = Const.DruidRepoopTime;
            }

            base.Update(gameTime);
        }

        protected void castLife(Vector2 loc)
        {
            if (Ready && DruidInfo.CurrentMana >= Const.LifeSpellMana)
            {
                LifeBlobInfo info = new LifeBlobInfo();
                info.Loc = loc;
                info.Amount = Const.LifeSpellAmount;
                LifeBlob blob = new LifeBlob(CursedGame, info);
                CursedGame.AddGob(blob);
                CoolDown += Const.LifeSpellCoolDown;
                DruidInfo.CurrentMana -= Const.LifeSpellMana;
            }
        }

        protected void castWood(Vector2 loc)
        {
            if (Ready && DruidInfo.CurrentMana >= Const.WoodSpellMana)
            {
                WoodInfo info = new WoodInfo();
                info.Loc = loc;
                info.Theta = (float)CursedGame.Random.NextDouble() * MathHelper.Pi * 2;
                Wood wood = new Wood(CursedGame, info);
                CursedGame.AddGob(wood);
                CoolDown += Const.WoodSpellCoolDown;
                DruidInfo.CurrentMana -= Const.WoodSpellMana;
            }
        }

        public override void control(ControlInfo cInfo)
        {
            base.control(cInfo);
            if (cInfo is MouseTrailInfo)
            {
                MouseTrailInfo mInfo = (MouseTrailInfo)cInfo;
                if (mInfo.Trail.Count > 0)
                {
                    Vector2 last = mInfo.Trail[mInfo.Trail.Count - 1];
                    if (mInfo.Type == MouseType.Offensive)
                        castLife(last);
                    if (mInfo.Type == MouseType.Defensive)
                        castWood(last);
                }
            }
            if (cInfo is MouseCursorInfo)
            {
                MouseCursorInfo mInfo = (MouseCursorInfo)cInfo;
                wand.GobInfo.Loc = mInfo.Mouse;
                if (mInfo.Clicked && mInfo.Type == MouseType.Offensive)
                {
                    castLife(mInfo.Mouse);
                    SimpleSound snd = new SimpleSound(CursedGame.SoundBank, CursedGame.SoundManager, lifeSound);
                    snd.play();
                }

                if (mInfo.Clicked && mInfo.Type == MouseType.Defensive)
                {
                    castWood(mInfo.Mouse);
                    SimpleSound snd = new SimpleSound(CursedGame.SoundBank, CursedGame.SoundManager, woodSound);
                    snd.play();
                }
            }
        }
        public override void takeDamage(float damage)
        {
            base.takeDamage(damage);
            VariableSound snd = new VariableSound(CursedGame.SoundBank,
               CursedGame.SoundManager, dmgSounds, CursedGame.Random);
            snd.play();
            #region temporary
            CursedGame.FogColor = new Microsoft.Xna.Framework.Graphics.Color((byte)255, CursedGame.FogColor.G, CursedGame.FogColor.B);
            #endregion
        }
        public override void cleanup()
        {
            CursedGame.RemoveGob(wand);
            base.cleanup();
        }
    }
}
