﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Cursed.gob.mob;
using System.Runtime.Serialization;

namespace Cursed.gob.tile
{
    public class FootSwitchInfo : WallInfo
    {
        public FootSwitchInfo(int x, int y)
            : base(x, y)
        {
            Label = "door";
            Depth = Const.FootSwitchDepth;
        }

        public override Gob create(CursedGame game)
        {
            return new FootSwitch(game, this);
        }
    }

    public class FootSwitch : Wall
    {
        public FootSwitch(CursedGame game, FootSwitchInfo info)
            : base(game, info)
        {
            Transcendent = true;
            renderInfo.Mask = Color.White;
            renderInfo.Texture = CursedGame.textureLib.GetTexture("switch");
        }

        public FootSwitchInfo SwitchInfo
        {
            get { return (FootSwitchInfo)info; }
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            base.Update(gameTime);
            bool triggered = false;
            foreach (Gob gob in touchedGobs)
            {
                if (gob is Mob && !gob.Transcendent)
                {
                    triggered = true;
                }
            }
            ICollection<Gob> labelGobs = CursedGame.getLabeledGobs(SwitchInfo.Label);
            foreach (Gob gob in labelGobs)
            {
                if (gob is Door)
                {
                    Door door = (Door)gob;
                    door.DoorInfo.Triggered = triggered;
                }
            }
        }
    }

    public class DoorInfo : WallInfo
    {
        bool open;
        bool triggered;

        public DoorInfo(int x, int y)
            : base(x, y)
        {
            open = false;
            triggered = false;
            Label = "door";
            Depth = Const.DoorDepth;
        }

        public override Gob create(CursedGame game)
        {
            return new Door(game, this);
        }

        public bool Triggered
        {
            get { return triggered; }
            set { triggered = value; }
        }

        public bool Open
        {
            get { return open; }
            set { open = value; }
        }
    }

    public class Door : Wall
    {
        public Door(CursedGame game, DoorInfo info)
            : base(game, info)
        {

        }

        public DoorInfo DoorInfo
        {
            get { return (DoorInfo)info; }
        }

        public override bool Transcendent
        {
            get { return DoorInfo.Open; }
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            base.Update(gameTime);

            bool occupied = false;
            foreach (Gob gob in touchedGobs)
            {
                if (gob is Mob && !gob.Transcendent)
                    occupied = true;
            }

            if (DoorInfo.Open && !DoorInfo.Triggered && !occupied)
                DoorInfo.Open = false;
            else if (DoorInfo.Triggered)
                DoorInfo.Open = true;

            if (DoorInfo.Open)
            {
                renderInfo.Mask = Color.Gray;
            }
            else
            {
                renderInfo.Mask = Color.Black;
            }
        }
    }
}
