﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.geom;
using Microsoft.Xna.Framework;
using System.Windows.Media;
using Cursed.view;
using Cursed.gob.mob;
using Cursed.input.ai;
using Cursed.view.renders;
using Cursed.gob.mob.kob;

namespace Cursed.gob.tile
{
    public class SpawnInfo : TileInfo
    {
        float spawnTime;
        int maxSpawns;

        public SpawnInfo(int x, int y)
            : base(x, y)
        {
            spawnTime = 5;
            maxSpawns = 3;
            Depth = Const.SpawnDepth;
        }

        public float SpawnTime
        {
            get { return spawnTime; }
            set { spawnTime = value; }
        }

        public int MaxSpawns
        {
            get { return maxSpawns; }
            set { maxSpawns = value; }
        }

    }

    public abstract class Spawn : Tile
    {
        float timeToSpawn;

        public Spawn(CursedGame game, SpawnInfo info)
            : base(game, info)
        {
            timeToSpawn = 0;

            shape = new Rect(Vector2.Zero, .5f, .5f);
            shape.setTransform(TileInfo.Loc, 0, 1);
            ((SpriteRenderInfo)renderInfo).Bounds = Bounds;
            renderInfo.Transform = Microsoft.Xna.Framework.Matrix.Identity;
            renderInfo.Mask = Microsoft.Xna.Framework.Graphics.Color.DarkOrange;

            Transcendent = true;
        }

        public SpawnInfo SpawnInfo
        {
            get { return (SpawnInfo)info; }
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);

            timeToSpawn -= gameTime.ElapsedGameTime.Milliseconds * .001f;
            if (timeToSpawn < 0)
            {
                timeToSpawn = SpawnInfo.SpawnTime;
                spawnCreature();
            }
        }

        protected abstract void spawnCreature();
    }

    public class DemonSpawnInfo : SpawnInfo
    {
        public DemonSpawnInfo(int x, int y)
            : base(x, y)
        {

        }
    }

    public class DemonSpawn : Spawn
    {
        public DemonSpawn(CursedGame game, DemonSpawnInfo info)
            : base(game, info)
        {

        }

        protected override void spawnCreature()
        {
            DemonInfo dmi = new DemonInfo();
            dmi.Loc = GobInfo.Loc;
            Demon dmn = new Demon(CursedGame, dmi);
            CursedGame.AddGob(dmn);
        }
    }
}
