﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Input;
using Cursed.view;
using Microsoft.Xna.Framework;
using Cursed.gob.mob;

namespace Cursed.input
{
    public class MouseController : Controller
    {
        ICamera camera;
        GameWindow window;

        MouseState mouseStateCurrent, mouseStatePrevious;
        Vector2 mousePosCurrent, mousePosPrevious;

        public MouseController(ICamera camera, GameWindow window)
            : base()
        {
            this.camera = camera;
            this.window = window;
            mouseStateCurrent = Mouse.GetState();
            mouseStatePrevious = Mouse.GetState();
        }

        public Vector2 CurrentMousePosition
        {
            get { return mousePosCurrent; }
        }

        public Vector2 LastMousePosition
        {
            get { return mousePosPrevious; }
        }

        protected void getMousePos()
        {
            mouseStatePrevious = mouseStateCurrent;
            mousePosPrevious = mousePosCurrent;

            mouseStateCurrent = Mouse.GetState();

            int x = mouseStateCurrent.X;
            int y = mouseStateCurrent.Y;

            int windowWidth = window.ClientBounds.Width;
            int windowHeight = window.ClientBounds.Height;
            float viewWidth = camera.ViewWidth;
            float viewHeight = camera.ViewHeight;

            float scaleX = viewWidth / windowWidth;
            float scaleY = viewHeight / windowHeight;

            x -= windowWidth / 2;
            y -= windowHeight / 2;

            float ox = x * scaleX;
            float oy = y * scaleY * -1;

            Vector2 pos = camera.Position;

            mousePosCurrent = new Vector2(pos.X + ox, pos.Y + oy);
            if (mousePosCurrent.X == float.NaN)
                mousePosCurrent.X = 0;
            if (mousePosCurrent.Y == float.NaN)
                mousePosCurrent.Y = 0;
        }

        public override void act(GameTime time)
        {
            getMousePos();
            MouseCursorInfo cInfo = new MouseCursorInfo();
            cInfo.Mouse = mousePosCurrent;
            cInfo.Clicked = false;
            if (mouseStateCurrent.LeftButton == ButtonState.Pressed && mouseStatePrevious.LeftButton == ButtonState.Released)
            {
                cInfo.Clicked = true;
                cInfo.Type = MouseType.Offensive;
            }
            if (mouseStateCurrent.RightButton == ButtonState.Pressed && mouseStatePrevious.RightButton == ButtonState.Released)
            {
                cInfo.Clicked = true;
                cInfo.Type = MouseType.Defensive;
            }
            if (Target != null)
                Target.control(cInfo);
        }

        public override void cleanup()
        {
            throw new NotImplementedException();
        }
    }
}
