﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Cursed.gob;
using Cursed.gob.mob;
using System.Windows;

namespace Cursed.view
{
    public class OrthoCamera : ICamera
    {
        GameWindow window;
        const float maxDepth = -50;
        const float minDepth = 50;
        Vector3 position;
        Matrix projection;
        Matrix view;
        float screenSize;
        float aspect;
        Stack<IFocus> focusStack;

        public OrthoCamera(float screenSize, GameWindow window)
        {
            this.screenSize = screenSize;
            this.window = window;
            focusStack = new Stack<IFocus>();
        }

        public void PushFocus(IFocus focus)
        {
            focusStack.Push(focus);
        }

        public bool PopFocus(IFocus focus)
        {
            if (Focus == focus)
            {
                focusStack.Pop();
                return true;
            }
            return false;
        }

        public IFocus Focus
        {
            get {
                if (focusStack.Count == 0)
                    return null;
                return focusStack.Peek(); 
            }
        }

        public Vector2 Position {
            get { 
                return new Vector2(position.X, position.Y);
            }
            set
            {
                position.X = value.X;
                position.Y = value.Y;
                position.Z = maxDepth;
                updateView(window);
            }
        }

        public Vector3 Position3
        {
            get { return position; }
            set { position = value; }
        }

        public float ScreenSize
        {
            get { return screenSize; }        
        }

        public float Aspect
        {
            get { return aspect; }
        }

        public float ViewWidth
        {
            get
            {
                if (aspect < 1)
                {
                    return screenSize;
                }
                return screenSize * aspect;
            }
        }

        public float ViewHeight
        {
            get
            {
                if (aspect < 1)
                {
                    return screenSize / aspect;
                }
                return screenSize;
            }
        }

        public void updateView(GameWindow window) {
            view = Matrix.CreateLookAt(new Vector3(position.X, position.Y, 0), position, Vector3.Up);
            aspect = (float)window.ClientBounds.Width / (float)window.ClientBounds.Height;
            if (aspect < 1.0f)
            {
                projection = Matrix.CreateOrthographic(screenSize, screenSize / aspect, minDepth, maxDepth);
            }
            else
            {
                projection = Matrix.CreateOrthographic(screenSize * aspect, screenSize, minDepth, maxDepth);
            }

           
        }

        public void trackFocus()
        {
            if (Focus != null)
            {
                Position = Focus.Loc;
            }
            else
            {
                Position = Vector2.Zero;
            }
        }

        public Matrix View
        {
            get { return view; }
        }

        public Matrix Projection
        {
            get { return projection; }
        }

        #region ICamera Members


        public Rect Bounds
        {
            get {
                float w = ViewWidth;
                float h = ViewHeight;
                float x = Position.X;
                float y = Position.Y;
                return new Rect(x - w / 2, y - h / 2, w, h);
            }
        }

        #endregion
    }
}
